/**
  ******************************************************************************
  * @file    Project/Internal_OSC_Calib/src/InternOscCalibration.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    02-August-2013
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2013 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software 
  * distributed under the License is distributed on an "AS IS" BASIS, 
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/** @addtogroup Internal_OSC_Calib
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
NVIC_InitTypeDef NVIC_InitStructureForCalib;
RCC_ClocksTypeDef RCC_ClocksForCalib;
TIM_ICInitTypeDef  TIM_ICInitStructure;
extern __IO uint32_t CaptureState;
extern __IO uint32_t Capture;
extern __IO uint32_t InternOscBeforeCalib, HSI14InternOscBeforeCalib;
extern __IO uint32_t StartCalibration , HSI14InternOscAfterCalib, InternOscAfterCalib;
__IO uint16_t SRC_ICBuffer[32]; 
__IO uint32_t iccapture1= 0,iccapture2 = 0, iccapture3 = 0;
__IO uint32_t AvrgValue = 0;
__IO uint32_t InputFrequency = 0;

/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Calibrates internal oscillators HSI to the minimum computed error.
  *         The system clock source is checked:
  *           - If HSI oscillator is used as system clock source, HSI is calibrated
  *             and the new HSI value is returned.
  *           - Otherwise function returns 0.
  * @param  None.
  * @retval The optimum computed frequency of HSI oscillator.
  *         Returning 0 means that the system clock source is not HSI.
  */
uint32_t HSI_CalibrateMinError(void)
{
  uint32_t measuredfrequency = 0;
  uint32_t optimumfrequency = 0;
  uint32_t frequencyerror = 0;
  uint32_t optimumfrequencyerror = 99999000; /* Large value */
  uint32_t numbersteps = 0;         /* Number of steps: size of trimming bits */
  uint8_t trimmingvalue = 0;
  uint8_t optimumcalibrationvalue = 0;
  uint8_t loopcounter = 0;
  
  /* Configure the GPIO ports before starting calibration process */
  GPIO_ConfigForCalibration();
  
  /* Configure clock before starting calibration process */
  CLK_ConfigForCalibration();
  
  /* Configure TIM14 before starting calibration process */
  TIM14_ConfigForCalibrationHSI();
  
  /* Get system clock frequency: value is in RCC_ClocksForCalib.SYSCLK_Frequency */
  RCC_GetClocksFreq(&RCC_ClocksForCalib);
  
  if (StartCalibration != 0)
  {
    /* HSITRIM is 5-bit length */
    numbersteps = 32; /* number of steps is 2^5 = 32 */
  }
  else
  {
    /* Without Calibration */
    numbersteps = 1;
  }
  
  /* Internal Osc frequency measurement for numbersteps */
  for (trimmingvalue = 0; trimmingvalue < numbersteps; trimmingvalue++)
  {
    if (StartCalibration != 0)
    {
      /* Set the Intern Osc trimming bits to trimmingvalue */
      RCC_AdjustCalibrationValue_HSI(RCC_GetSYSCLKSource(), trimmingvalue);
    }

    /* Start frequency measurement for current trimming value */
    measuredfrequency = 0;
    loopcounter = 0;
    /************ Start measuring Internal Oscillator frequency ***************/
    while (loopcounter <= NUMBER_OF_LOOPS)
    {
      CaptureState = CAPTURE_START;
      
      /* Generate update */
      TIM_GenerateEvent(TIM14, TIM_EventSource_Update);

      /* Clear all TM14 flags */
      TIM14->SR = 0;

      /* Enable capture 1 interrupt */
      TIM_ITConfig(TIM14, TIM_IT_CC1, ENABLE);

      /* Enable TIM14 counter */
      TIM_Cmd(TIM14, ENABLE);

      /* Enable the TIM14 IRQ channel */
      NVIC_InitStructureForCalib.NVIC_IRQChannelCmd = ENABLE;
      NVIC_Init(&NVIC_InitStructureForCalib);

      /* Wait for end of capture: two consecutive captures */
      while(CaptureState != CAPTURE_COMPLETED);

      /* Disable IRQ channel */
      NVIC_InitStructureForCalib.NVIC_IRQChannelCmd = DISABLE;
      NVIC_Init(&NVIC_InitStructureForCalib);

      /* Disable TIM14 */
      TIM_Cmd(TIM14, DISABLE);

      if (loopcounter != 0)
      {
        /* Compute the frequency (the Timer prescaler isn't included) */
        measuredfrequency += (uint32_t) (REFERENCE_FREQUENCY * Capture);
      }

      /* Increment loop counter */
      loopcounter++;
    }
    /**************************** END of Measurement **************************/

    /* Compute the average value corresponding the the current trimming value */
    measuredfrequency = (uint32_t)((TIM_GetPrescaler(TIM14) + 1) * (measuredfrequency / NUMBER_OF_LOOPS));
    
    /* Get The HSI14 value without calibration */
    if(trimmingvalue == 16)
    {
      InternOscBeforeCalib = measuredfrequency;
    }
    
    if(StartCalibration != 0)
    {
      /* Compute current frequency error corresponding to the current trimming value:
      measured value is subtracted from the typical one */
      frequencyerror = ABS_RETURN((int32_t) (measuredfrequency - RCC_ClocksForCalib.SYSCLK_Frequency));
      
      /* Get the nearest frequency value to typical one */
      if (optimumfrequencyerror > frequencyerror)
      {
        optimumfrequencyerror = frequencyerror;
        optimumcalibrationvalue = trimmingvalue;
        optimumfrequency = measuredfrequency;
      }
    } 
  }
  
  if(StartCalibration != 0)
  {
    /* Set trimming bits corresponding to the nearest frequency */
    RCC_AdjustCalibrationValue_HSI(RCC_GetSYSCLKSource(), optimumcalibrationvalue);
    /* Return the intern oscillator frequency after calibration */
    return (optimumfrequency);
  }
  else
  {
    /* Return the intern oscillator frequency before calibration */
    return (measuredfrequency);
  }  
}

/**
  * @brief  Calibrates the internal oscillator (HSI only) with the maximum allowed 
  *         error value set by user.
  *         If this value was not found, this function sets the oscillator
  *         to default value.
  * @param  MaxAllowedError: maximum absolute value allowed of the HSI frequency 
  *                          error given in Hz.
  * @param  Freq: pointer to an uint32_t variable that will contain the value  
  *               of the internal oscillator frequency after calibration.
  * @retval ErrorStatus:
  *            - SUCCESS: a frequency error =< MaxAllowedError was found.
  *            - ERROR: a frequency error =< MaxAllowedError was not found.
  */
ErrorStatus HSI_CalibrateFixedError(uint32_t MaxAllowedError, uint32_t* Freq)
{
  uint32_t measuredfrequency;
  uint32_t frequencyerror = 0;
  uint8_t loopcounter = 0;
  uint8_t trimmingindex = 0;
  uint8_t trimmingvalue = 16;
  int8_t Sign = 1;
  ErrorStatus calibrationstatus = ERROR;

  SetSysClock_HSI_8MHz();
  
  /* Configure the GPIO ports before starting calibration process */
  GPIO_ConfigForCalibration();
  
  /* Configure clock before starting calibration process */
  CLK_ConfigForCalibration();

  /* Configure TIM14 before starting calibration process */
  TIM14_ConfigForCalibrationHSI();

  /* Get system clock frequency: value is in RCC_ClocksForCalib.SYSCLK_Frequency */
  RCC_GetClocksFreq(&RCC_ClocksForCalib);

  /****************************************************************************/
     /* Set the HSITRIMR register to trimmingvalue to be ready for measurement */
    RCC_AdjustHSICalibrationValue(16);

    /* Start frequency measurement for current trimming value */
    measuredfrequency = 0;
    loopcounter = 0;

    /************ Start measuring Internal Oscillator frequency ***************/
    while (loopcounter <= NUMBER_OF_LOOPS)
    {
      CaptureState = CAPTURE_START;

      /* Generate update */
      TIM_GenerateEvent(TIM14, TIM_EventSource_Update);

      /* Clear all TM10 flags */
      TIM14->SR = 0;

      /* Enable capture 1 interrupt */
      TIM_ITConfig(TIM14, TIM_IT_CC1, ENABLE);

      /* Enable TIM */
      TIM_Cmd(TIM14, ENABLE);
      NVIC_InitStructureForCalib.NVIC_IRQChannelCmd = ENABLE;
      NVIC_Init(&NVIC_InitStructureForCalib);

      /* Wait for end of capture: two consecutive captures */
      while(CaptureState != CAPTURE_COMPLETED);

      /* Disable IRQ channel */
      NVIC_InitStructureForCalib.NVIC_IRQChannelCmd = DISABLE;
      NVIC_Init(&NVIC_InitStructureForCalib);

      /* Disable TIM */
      TIM_Cmd(TIM14, DISABLE);

      if (loopcounter != 0)
      {
        /* Compute the frequency (the Timer prescaler isn't included) */
        /* Then add the current frequency to previous cumulation */
        InternOscBeforeCalib += (uint32_t) (REFERENCE_FREQUENCY * Capture);
      }

      /* Increment loop counter */
      loopcounter++;
    }
    /**************************** END of Measurement **************************/
    /* Compute current frequency Error corresponding to the current HSITRIM value */
    /* Compute the average value corresponding the the current trimming value */
    InternOscBeforeCalib = (uint32_t)((TIM_GetPrescaler(TIM14) + 1) * (InternOscBeforeCalib / NUMBER_OF_LOOPS));

  /****************************************************************************/
  
  
  /* RC Frequency measurement for different values */
  for(trimmingindex = 0; trimmingindex < 32; trimmingindex++)
  {
    /* Compute trimming value */
    trimmingvalue = trimmingvalue + (trimmingindex * Sign);
    Sign *= (-1);

    /* Set the HSITRIMR register to trimmingvalue to be ready for measurement */
    RCC_AdjustHSICalibrationValue(trimmingvalue);

    /* Start frequency measurement for current trimming value */
    measuredfrequency = 0;
    loopcounter = 0;

    /************ Start measuring Internal Oscillator frequency ***************/
    while (loopcounter <= NUMBER_OF_LOOPS)
    {
      CaptureState = CAPTURE_START;

      /* Generate update */
      TIM_GenerateEvent(TIM14, TIM_EventSource_Update);

      /* Clear all TM10 flags */
      TIM14->SR = 0;

      /* Enable capture 1 interrupt */
      TIM_ITConfig(TIM14, TIM_IT_CC1, ENABLE);

      /* Enable TIM */
      TIM_Cmd(TIM14, ENABLE);
      NVIC_InitStructureForCalib.NVIC_IRQChannelCmd = ENABLE;
      NVIC_Init(&NVIC_InitStructureForCalib);

      /* Wait for end of capture: two consecutive captures */
      while(CaptureState != CAPTURE_COMPLETED);

      /* Disable IRQ channel */
      NVIC_InitStructureForCalib.NVIC_IRQChannelCmd = DISABLE;
      NVIC_Init(&NVIC_InitStructureForCalib);

      /* Disable TIM */
      TIM_Cmd(TIM14, DISABLE);
      
      if (loopcounter != 0)
      {
        /* Compute the frequency (the Timer prescaler isn't included) */
        /* Then add the current frequency to previous cumulation */
        measuredfrequency += (uint32_t) (REFERENCE_FREQUENCY * Capture);
      }

      /* Increment loop counter */
      loopcounter++;
    }
    /**************************** END of Measurement **************************/
    /* Compute current frequency Error corresponding to the current HSITRIM value */
    /* Compute the average value corresponding the the current trimming value */
    measuredfrequency = (uint32_t)((TIM_GetPrescaler(TIM14) + 1) * (measuredfrequency / NUMBER_OF_LOOPS));

    /* Compute current frequency error corresponding to the current trimming value:
       measured value is subtracted from the typical one */
    frequencyerror = ABS_RETURN((int32_t) (measuredfrequency - RCC_ClocksForCalib.SYSCLK_Frequency));

    /* Check if frequency error is less or equal to value set by the user */
    if(frequencyerror <= MaxAllowedError)
    {
      calibrationstatus = SUCCESS; /* The calibration has succeed */
      break; /* stop searching and measurements for frequencies */
    }
  }

  /* Save the HSI measured value */
  *Freq = measuredfrequency;

  /* If the frequency error set by the user was not found */
  if(calibrationstatus == ERROR)
  {
    /* Set the HSITRIMR register to default value */
     RCC_AdjustHSICalibrationValue(16);
  }

  /* Return the calibration status: ERROR or SUCCESS */
  return (calibrationstatus);
}

/**
  * @brief  Adjust calibration value (writing to trimming bits) of selected oscillator.
  * @param  InternOsc: Internal Oscillator source: HSI
  * @param  TrimmingValue: calibration value to be written in trimming bits.
  * @retval None.
  */
void RCC_AdjustCalibrationValue_HSI(uint8_t InternOsc, uint8_t TrimmingValue)
{
  RCC_AdjustHSICalibrationValue(TrimmingValue);
}

/**
  * @brief Configures the TIM14 in input capture to measure HSI frequency.
  * @param  None.
  * @retval None.
  */
void TIM14_ConfigForCalibrationHSI(void)
{
  TIM_ICInitTypeDef  TIM_ICInitStructure;

  /* Enable TIM14 clock */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM14, ENABLE);

  /* Reset TIM14 registers */
  TIM_DeInit(TIM14);

  /* Configure TIM14 prescaler */
  TIM_PrescalerConfig(TIM14, TIM14_COUNTER_PRESCALER, TIM_PSCReloadMode_Immediate);

#ifdef USE_REFERENCE_LSE
  /* Connect RTC clock (.i.e LSE clock) to TIM14 Input Capture 1 */
  TIM_RemapConfig(TIM14, TIM14_RTC_CLK);
#else
  /* TIM14 Channel 1 is connected to GPIO clock */
  TIM_RemapConfig(TIM14, TIM14_GPIO);
#endif /* USE_REFERENCE_LSE */

  /* TIM14 configuration: Input Capture mode ---------------------
     The reference clock(LSE or external) is connected to TIM14 CH1
     The Rising edge is used as active edge,
     The TIM14 CCR1 is used to compute the frequency value 
  ------------------------------------------------------------ */
  TIM_ICInitStructure.TIM_Channel     = TIM_Channel_1;
  TIM_ICInitStructure.TIM_ICPolarity  = TIM_ICPolarity_Rising;
  TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
  TIM_ICInitStructure.TIM_ICPrescaler = TIM14_IC_DIVIDER;
  TIM_ICInitStructure.TIM_ICFilter = 0x0;
  TIM_ICInit(TIM14, &TIM_ICInitStructure);

  /* Enable the TIM14 global Interrupt */
  NVIC_InitStructureForCalib.NVIC_IRQChannel = TIM14_IRQn;
  NVIC_InitStructureForCalib.NVIC_IRQChannelPriority = 0;
  NVIC_InitStructureForCalib.NVIC_IRQChannelCmd = DISABLE;
  NVIC_Init(&NVIC_InitStructureForCalib);
}

/**
  * @brief  Calibrates internal oscillators HSI14 to the minimum computed error.
  *         The system clock source is checked:
  *           - If HSI oscillator is used as system clock source, HSI14 is calibrated
  *             and the new HSI14 value is returned.
  *           - Otherwise function returns 0.
  * @param  None.
  * @retval The optimum computed frequency of HSI14 oscillator.
  *         Returning 0 means that the system clock source is no HSI.
  */
uint32_t HSI14_CalibrateMinError(void)
{
  __IO uint32_t measuredfrequency = 0;
  uint32_t optimumfrequency = 0;
  uint32_t frequencyerror = 0;
  uint8_t optimumcalibrationvalue = 0;
  uint32_t optimumfrequencyerror = 99999000; /* Large value */
  uint8_t trimmingvalue = 0;
  uint8_t loopcounter = 0;

  /* Set HSI as PLL source clock and set the system clock to 48Mhz */
  SetSysClock_HSI_48MHz();
  
  /* TIM14 De-init */
  TIM_DeInit(TIM14);
  
  /* TIM Configuration to measure the HSI 14 */
  TIM14_ConfigForCalibrationHSI14();
   
  /* Get system clock frequency: value is in RCC_ClocksForCalib.SYSCLK_Frequency */
  RCC_GetClocksFreq(&RCC_ClocksForCalib);
  
  /* Internal Osc frequency measurement for numbersteps */
  for (trimmingvalue = 0; trimmingvalue < 32 ; trimmingvalue++)
  {   
    /* Set the Intern Osc trimming bits to trimmingvalue */
    RCC_AdjustCalibrationValue_HSI14(RCC_GetSYSCLKSource(), trimmingvalue);
    
    /* Start frequency measurement for current trimming value.
       This code uses direct register access to have fast access to the Timer IC 
       value. 
       */
    measuredfrequency = 0;
    
    loopcounter = 0;

    /* Clear TIM14 flags */
    TIM14->SR = 0;
    
    /* TIM14 enable counter */
    TIM_Cmd(TIM14, ENABLE);
    
    iccapture1 = TIM14->CCR1;    
    while(TIM14->CCR1 == iccapture1);  

    iccapture1 = TIM14->CCR1;
    iccapture2 = iccapture1;
      
    while(loopcounter < NUMBER_OF_LOOPS)
    {
      while(TIM14->CCR1 == iccapture2);    

      iccapture2 = TIM14->CCR1;      
      loopcounter++;
    }
     
    /* TIM14 disable counter */
    TIM_Cmd(TIM14, DISABLE);
    
    /**************************** END of Measurement **************************/
    /* Compute the average value */
    iccapture3 = ((1000 * (iccapture2 - iccapture1))/ NUMBER_OF_LOOPS);
    
    /* Compute the HSI14 frequency value 

       HSI14 period = TIM IC value / (System clock * TIM prescaler)
         knowing that:
           System clock = (HSI /2 ) * 12 = (InternOscAfterCalib /2 ) * 12
           TIM prescaler = 8 
       so, HSI14 period = TIM IC value / (48 * InternOscAfterCalib
       and HSI14 frequency = (48 * InternOscAfterCalib) / TIM IC value          
       */       
    measuredfrequency = 1000 * ((48 * InternOscAfterCalib) / iccapture3);
    
    /* Get The HSI14 value without calibration */
    if(trimmingvalue == 16)
    {
      /* HSI 14 Value before calibration */
      HSI14InternOscBeforeCalib = measuredfrequency;
    }
    
    /* Compute current frequency error corresponding to the current trimming value:
    measured value is subtracted from the typical one */
    frequencyerror = ABS_RETURN((int32_t) (measuredfrequency - HSI14_VALUE));
    
    /* Get the nearest frequency value to typical one */
    if (optimumfrequencyerror > frequencyerror)
    {
      optimumfrequencyerror = frequencyerror;
      optimumcalibrationvalue = trimmingvalue;
      optimumfrequency = measuredfrequency;
    }   
  }
   
  /* Set trimming bits corresponding to the nearest frequency */
  RCC_AdjustCalibrationValue_HSI14(RCC_GetSYSCLKSource(), optimumcalibrationvalue);
  
  /* Return the intern oscillator frequency after calibration */
  return (optimumfrequency);  
}

/**
  * @brief  Calibrates the internal oscillator (HSI14) with the maximum allowed 
  *         error value set by user.
  *         If this value was not found, this function sets the oscillator
  *         to default value.
  * @param  MaxAllowedError: maximum absolute value allowed of the HSI14 frequency 
  *                          error given in Hz.
  * @param  Freq: pointer to an uint32_t variable that will contain the value  
  *               of the internal oscillator frequency after calibration.
  * @retval ErrorStatus:
  *            - SUCCESS: a frequency error =< MaxAllowedError was found.
  *            - ERROR: a frequency error =< MaxAllowedError was not found.
  */
ErrorStatus HSI14_CalibrateFixedError(uint32_t MaxAllowedError, uint32_t* Freq)
{
  uint32_t measuredfrequency;
  uint32_t frequencyerror = 0;
  uint8_t trimmingindex = 0;
  uint8_t trimmingvalue = 16;
  int8_t Sign = 1;
  ErrorStatus calibrationstatus = ERROR;
  uint8_t loopcounter = 0;

  /* Set HSI as PLL source clock and set the system clock to 48Mhz */
  SetSysClock_HSI_48MHz();
  
  /* TIM14 De-init */
  TIM_DeInit(TIM14);
  
  /* TIM Configuration to measure the HSI 14 */
  TIM14_ConfigForCalibrationHSI14();
   
  /* Get system clock frequency: value is in RCC_ClocksForCalib.SYSCLK_Frequency */
  RCC_GetClocksFreq(&RCC_ClocksForCalib);
  
  /* Set the Intern Osc trimming bits to trimmingvalue */
  RCC_AdjustCalibrationValue_HSI14(RCC_GetSYSCLKSource(), 16);
    
  /* Internal Osc frequency measurement for numbersteps */
  for (trimmingvalue = 0; trimmingvalue < 32 ; trimmingvalue++)
  {
    
    /* Compute trimming value */
    trimmingvalue = trimmingvalue + (trimmingindex * Sign);
    Sign *= (-1);
    
    /* Set the Intern Osc trimming bits to trimmingvalue */
    RCC_AdjustCalibrationValue_HSI14(RCC_GetSYSCLKSource(), trimmingvalue);
    
   /* Start frequency measurement for current trimming value ******************/
    loopcounter = 0;
    iccapture1 = 0;
    iccapture2 = 0;
    iccapture3 = 0;

    /* Clear TIM14 flags */
    TIM14->SR = 0;
    
    /* TIM14 enable counter */
    TIM_Cmd(TIM14, ENABLE);
    
    iccapture1 = TIM14->CCR1;    
    while(TIM14->CCR1 == iccapture1);  

    iccapture1 = TIM14->CCR1;
    iccapture2 = iccapture1;
      
    while(loopcounter < NUMBER_OF_LOOPS)
    {
      while(TIM14->CCR1 == iccapture2);    

      iccapture2 = TIM14->CCR1;      
      loopcounter++;
    }   
     
    /* TIM14 disable counter */
    TIM_Cmd(TIM14, DISABLE);
    
    /**************************** END of Measurement **************************/
    /* Compute the average value */
    iccapture3 = ((1000 * (iccapture2 - iccapture1))/ NUMBER_OF_LOOPS);
    
    /* Compute the HSI14 frequency value 

       HSI14 period = TIM IC value / (System clock * TIM prescaler)
         knowing that:
           System clock = (HSI /2 ) * 12 = (InternOscAfterCalib /2 ) * 12
           TIM prescaler = 8 
       so, HSI14 period = TIM IC value / (48 * InternOscAfterCalib
       and HSI14 frequency = (48 * InternOscAfterCalib) / TIM IC value          
       */
    measuredfrequency = 1000 * ((48 * InternOscAfterCalib) / iccapture3);
    
    /* Get The HSI14 value without calibration */
    if(trimmingvalue == 16)
    {
      HSI14InternOscBeforeCalib = measuredfrequency;
    }
    
    /* Compute current frequency error corresponding to the current trimming value:
    measured value is subtracted from the typical one */
    frequencyerror = ABS_RETURN((int32_t) (measuredfrequency - HSI14_VALUE));
    
    /* Check if frequency error is less or equal to value set by the user */
    if(frequencyerror <= MaxAllowedError)
    {
      calibrationstatus = SUCCESS; /* The calibration has succeed */
      break; /* stop searching and measurement of frequencies */
    }
  }
  
  /* Save the HSI measured value */
  *Freq = measuredfrequency;

  /* If the frequency error set by the user was not found */
  if(calibrationstatus == ERROR)
  {
    /* Set the HSITRIMR register to default value */
    RCC_AdjustCalibrationValue_HSI14(RCC_GetSYSCLKSource(), 16);
  }

  /* Return the calibration status: ERROR or SUCCESS */
  return (calibrationstatus);
}



/**
  * @brief  Adjust calibration value (writing to trimming bits) of selected oscillator.
  * @param  InternOsc: Internal Oscillator source: HSI14
  * @param  TrimmingValue: calibration value to be written in trimming bits.
  * @retval None.
  */
void RCC_AdjustCalibrationValue_HSI14(uint8_t InternOsc, uint8_t TrimmingValue)
{
  RCC_AdjustHSI14CalibrationValue(TrimmingValue);
}

/**
  * @brief Configures the TIM14 in input capture to measure HSI frequency.
  * @param  HSI value after CAlibration .
  * @retval None.
  */
void TIM14_ConfigForCalibrationHSI14(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  
  /* TIM14 clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM14, ENABLE);

  /* GPIOA clock enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
  
  /* TIM14 channel 1 pin (PA4) configuration */
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_4;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  /* Connect TIM pins to AF4 */
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource4, GPIO_AF_4);

  /* MCO pin configuration: PA8 */
  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_8;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource8, GPIO_AF_0);
  /* Output HSI14 on MCO pin */
  RCC_MCOConfig(RCC_MCOSource_HSI14);  
  
  /* Enable the HSI14 */
  RCC_HSI14Cmd(ENABLE);
  
  /* TIM1 configuration: Input Capture mode ---------------------
  The intrenal signal is connected to TIM14 CH1 pin (PA4)  
  The Rising edge is used as active edge,
  The TIM14 CCR1 is used to compute the frequency value 
  ------------------------------------------------------------ */
  TIM_ICInitStructure.TIM_Channel = TIM_Channel_1;
  TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
  TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
  TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV8;
  TIM_ICInitStructure.TIM_ICFilter = 0x0;
  TIM_ICInit(TIM14, &TIM_ICInitStructure);
  
  /* TIM14_TI1 mapped on MCO (MCO output is HSI14) */
  TIM_RemapConfig(TIM14, TIM14_MCO);  
}

/**
  * @brief  Configures System clock to 8 MHz, clocked by HSI.
  * @note   This function assumes that the HSI is already enabled   
  * @param  None.
  * @retval None.
  */
void SetSysClock_HSI_8MHz(void)
{
  /* At this stage the HSI is already enabled --------------------------------*/

  /* Enable Prefetch Buffer and Flash 0 wait state */
  FLASH->ACR = FLASH_ACR_PRFTBE;

  /* HCLK = SYSCLK / 1 */
  RCC->CFGR |= (uint32_t)RCC_CFGR_HPRE_DIV1;
       
  /* PCLK = HCLK / 1 */
  RCC->CFGR |= (uint32_t)RCC_CFGR_PPRE_DIV1;
           
  /* Select HSI as system clock source */
  RCC->CFGR &= (uint32_t)((uint32_t)~(RCC_CFGR_SW));   

  /* Wait till HSI is used as system clock source */
  while ((RCC->CFGR & (uint32_t)RCC_CFGR_SWS) != (uint32_t)RCC_CFGR_SWS_HSI)
  {
  }
}

/**
  * @brief  Configures System clock to 48 MHz. PLL (clocked by HSI) is used as 
  *         System clock source
  * @note   This function assumes that the HSI is already enabled   
  * @param  None.
  * @retval None.
  */
void SetSysClock_HSI_48MHz(void)
{
  /* At this stage the HSI is already enabled --------------------------------*/

  /* Enable Prefetch Buffer and Flash 1 wait state */
  FLASH->ACR = FLASH_ACR_PRFTBE | FLASH_ACR_LATENCY;

  /* HCLK = SYSCLK / 1 */
  RCC->CFGR |= (uint32_t)RCC_CFGR_HPRE_DIV1;
       
  /* PCLK = HCLK / 1 */
  RCC->CFGR |= (uint32_t)RCC_CFGR_PPRE_DIV1;

  /* PLL output = (HSI /2) * 12 = 48 MHz */
  RCC->CFGR &= (uint32_t)((uint32_t)~(RCC_CFGR_PLLSRC | RCC_CFGR_PLLXTPRE | RCC_CFGR_PLLMULL));
  RCC->CFGR |= (uint32_t)(RCC_CFGR_PLLSRC_HSI_Div2 | RCC_CFGR_PLLXTPRE_PREDIV1 | RCC_CFGR_PLLMULL12);
            
  /* Enable PLL */
  RCC->CR |= RCC_CR_PLLON;

  /* Wait till PLL is ready */
  while((RCC->CR & RCC_CR_PLLRDY) == 0)
  {
  }

  /* Select PLL as system clock source */
  RCC->CFGR &= (uint32_t)((uint32_t)~(RCC_CFGR_SW));
  RCC->CFGR |= (uint32_t)RCC_CFGR_SW_PLL;    

  /* Wait till PLL is used as system clock source */
  while ((RCC->CFGR & (uint32_t)RCC_CFGR_SWS) != (uint32_t)RCC_CFGR_SWS_PLL)
  {
  }
}

/**
  * @brief  Configures LSE to be used as RTC clock source
  * @param  None.
  * @retval None.
  */
void CLK_ConfigForCalibration(void)
{
#ifdef USE_REFERENCE_LSE
  /* Enable the PWR clock */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);
  
  /* Allow access to BKP Domain */
  PWR_BackupAccessCmd(ENABLE);
  
  /* Reset RTC Domain */
  RCC_RTCCLKCmd(ENABLE);
  RCC_RTCCLKCmd(DISABLE);
  
  RCC_BackupResetCmd(ENABLE);
  RCC_BackupResetCmd(DISABLE);
  
  /* Enable the LSE OSC */
  RCC_LSEConfig(RCC_LSE_ON);
  
  /* Wait till LSE is ready */  
  while(RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET)
  {
  }
  
  /* Select the RTC Clock Source */
  RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);
   
  /* Enable the RTC Clock */
  RCC_RTCCLKCmd(ENABLE);

  /* Wait for RTC APB registers synchronisation */
  RTC_WaitForSynchro();

#endif /* USE_REFERENCE_LSE */
}

/**
  * @brief Configures the GPIO: configure each used I/O in the suitable setting
  * @param None.
  * @retval None.
  */
void GPIO_ConfigForCalibration(void)
{
#ifndef USE_REFERENCE_LSE

  GPIO_InitTypeDef GPIO_InitStructure;
  
    /* GPIOA clock enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
  
  /* TIM14 channel 1 pin (PA4) configuration */
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_4;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  /* Connect TIM pins to AF4 */
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource4, GPIO_AF_4);
#endif /* USE_REFERENCE_LSE */
  
}

/**
  * @}
  */ 

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
