/**
  ******************************************************************************
  * @file    Project/Internal_OSC_Calib/src/LSIMeasurement.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    02-August-2013
  * @brief   This file provides all the LSI Measurement firmware functions.
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2013 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software 
  * distributed under the License is distributed on an "AS IS" BASIS, 
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/** @addtogroup Internal_OSC_Calib
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
NVIC_InitTypeDef NVIC_InitStructureForLSI;

/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
extern __IO uint32_t CaptureState;
extern __IO uint32_t Capture;

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Measures the LSI clock frequency using TIM14 capture interrupt.
  * @param  InternOscFrequency: The system clock source value.
  *         In case of using HSI oscillator as system clock source:
  *           InternOscFrequency should be 8000000 (or a more accurate value).
  *         In case of using HSI oscillator as system clock source:
  *           InternOscFrequency should correspond to the selected range (or a more accurate value).
  * @retval The LSI frequency value.
  */
uint16_t LSI_FreqMeasure(uint32_t InternOscFrequency)
{
  uint32_t measuredfrequency = 0;
  uint8_t loopcounter = 0;
  
  SetSysClock_HSI_8MHz();
  
  /* Configure CLK for LSI measurement process */
  CLK_ConfigForLSI();

  /* Configure TIM14 for LSI measurement process */
  TIM14_ConfigForLSI();

  /* Reset counter */
  loopcounter = 0;
  
  /**************************** START of LSI Measurement **********************/
  while (loopcounter <= LSI_PERIOD_NUMBERS)
  {
    CaptureState = CAPTURE_START;

    /* Generate update */
    TIM_GenerateEvent(TIM14, TIM_EventSource_Update);

    /* Clear all TM10 flags */
    TIM14->SR = 0;

    /* Enable capture 1 interrupt */
    TIM_ITConfig(TIM14, TIM_IT_CC1, ENABLE);

    /* Enable TIM */
    TIM_Cmd(TIM14, ENABLE);

    /* Enable the TIM14 global Interrupt */
    NVIC_InitStructureForLSI.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructureForLSI);

    /* Wait for end of capture: two consecutive captures */
    while(CaptureState != CAPTURE_COMPLETED);

    /* Disable IRQ channel */
    NVIC_InitStructureForLSI.NVIC_IRQChannelCmd = DISABLE;
    NVIC_Init(&NVIC_InitStructureForLSI);
    
    /* Disable TIM14 */
    TIM_Cmd(TIM14, DISABLE);

    if (loopcounter != 0)
    {
      /* Compute the frequency value: multiplying by 8 is due to TIM_ICPSC_DIV8 */
      /* Then add the current frequency to previous cumulation */
      measuredfrequency += (uint32_t) 8 * (InternOscFrequency / Capture);
    }
    
    /* Increment counter */
    loopcounter++;
  }
  /**************************** END of LSI Measurement ************************/

  /* Compute the average of LSI frequency value */
  measuredfrequency = (uint16_t) (measuredfrequency / LSI_PERIOD_NUMBERS);

  /* Return the LSI frequency */
  return (uint16_t)(measuredfrequency);
}

/**
  * @brief  Configures LSI to be used as RTC clock source
  * @param  None.
  * @retval None.
  */
void CLK_ConfigForLSI(void)
{
  /* Enable the PWR clock */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);
  
  /* Allow access to BKP Domain */
  PWR_BackupAccessCmd(ENABLE);
  
  /* Reset RTC Domain */
  RCC_RTCCLKCmd(ENABLE);
  RCC_RTCCLKCmd(DISABLE);
  
  RCC_BackupResetCmd(ENABLE);
  RCC_BackupResetCmd(DISABLE);
  
  /* LSI used as RTC source clock */
  /* The RTC Clock may varies due to LSI frequency dispersion. */   
  /* Enable the LSI OSC */ 
  RCC_LSICmd(ENABLE);
  
  /* Wait till LSI is ready */  
  while(RCC_GetFlagStatus(RCC_FLAG_LSIRDY) == RESET)
  {
  }

  /* Select the RTC Clock Source */
  RCC_RTCCLKConfig(RCC_RTCCLKSource_LSI);
   
  /* Enable the RTC Clock */
  RCC_RTCCLKCmd(ENABLE);

  /* Wait for RTC APB registers synchronisation */
  RTC_WaitForSynchro();  
}

/**
  * @brief  Configures the TIM 14 channel 1 in input capture to measure the LSI freqeuncy.
  * @param  None.
  * @retval None.
  */
void TIM14_ConfigForLSI(void)
{
  TIM_ICInitTypeDef  TIM_ICInitStructure;

  /* Enable TIM14 clock */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM14, ENABLE);

  /* Reset TIM14 registers */
  TIM_DeInit(TIM14);

  /* Configure TIM14 prescaler */
  TIM_PrescalerConfig(TIM14, 0, TIM_PSCReloadMode_Immediate);

  /* Connect RTC clock (.i.e LSI clock) to TIM14 Input Capture 1 */
  TIM_RemapConfig(TIM14, TIM14_RTC_CLK);

  /* TIM14 configuration: Input Capture mode ---------------------
     The LSI clock is connected to TIM14 CH1
     The Rising edge is used as active edge,
     The TIM14 CCR1 is used to compute the frequency value 
  ------------------------------------------------------------ */
  TIM_ICInitStructure.TIM_Channel     = TIM_Channel_1;
  TIM_ICInitStructure.TIM_ICPolarity  = TIM_ICPolarity_Rising;
  TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
  TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV8;
  TIM_ICInitStructure.TIM_ICFilter = 0x0;
  TIM_ICInit(TIM14, &TIM_ICInitStructure);

  /* Enable the TIM14 global Interrupt */
  NVIC_InitStructureForLSI.NVIC_IRQChannel = TIM14_IRQn;
  NVIC_InitStructureForLSI.NVIC_IRQChannelPriority = 0;
  NVIC_InitStructureForLSI.NVIC_IRQChannelCmd = DISABLE;
  NVIC_Init(&NVIC_InitStructureForLSI);
}

/**
  * @}
  */ 

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
