/**
  ******************************************************************************
  * @file    Project/Internal_OSC_Calib/inc/main.c 
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    02-August-2013
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2013 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software 
  * distributed under the License is distributed on an "AS IS" BASIS, 
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/** @addtogroup Internal_OSC_Calib
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Enable this define to use the minimum error method to calibrate the HSI and
   HSI14 oscillators */
#define CALIBRATION_MIN_ERROR

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
static __IO uint32_t TimingDelay;
RCC_ClocksTypeDef RCC_Clocks;

__IO uint32_t InternOscAfterCalib = 0, HSI14InternOscBeforeCalib = 0,InternOscBeforeCalib = 0;
__IO uint32_t HSI14InternOscAfterCalib,StartCalibration = 0;
__IO uint16_t LSIFrequency = LSI_VALUE;
__IO uint32_t CurrentHSIFrequency = 0;

/* Private function prototypes -----------------------------------------------*/
void Delay(__IO uint32_t nTime);
static void LCD_Display(void);

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
  /*!< At this stage the microcontroller clock setting is already configured, 
       this is done through SystemInit() function which is called from startup
       file (startup_stm32f0xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_stm32f0xx.c file
     */ 
  
  /* SysTick end of count event each 10ms */
  RCC_GetClocksFreq(&RCC_Clocks);
  SysTick_Config(RCC_Clocks.HCLK_Frequency / 100);

  /* Display message on STM320518-EVAL LCD ************************************/
  STM320518_LCD_Init();
  /* Clear the LCD */ 
  LCD_Clear(LCD_COLOR_WHITE);

  /* Set the LCD Back Color */
  LCD_SetBackColor(LCD_COLOR_BLUE);

  /* Set the LCD Text Color */
  LCD_SetTextColor(LCD_COLOR_WHITE);
  LCD_DisplayStringLine(LCD_LINE_0, (uint8_t*)" Inter. Oscillators ");
  LCD_DisplayStringLine(LCD_LINE_1, (uint8_t*)"     Calibration    ");

  /* Turn on leds available on STM320518-EVAL *********************************/
  STM_EVAL_LEDInit(LED1);
  STM_EVAL_LEDInit(LED2);
  
  STM_EVAL_LEDOn(LED1);
  STM_EVAL_LEDOn(LED2);

  
  /************************** Start the Calibration ***************************/
  /* Enable Calibration */
  StartCalibration = 1;
  
#ifdef CALIBRATION_MIN_ERROR 
  /* Calibrates internal oscillators HSI to the minimum computed error */
  InternOscAfterCalib = HSI_CalibrateMinError();
  
  /* Calibrates internal oscillators HSI14 to the minimum computed error */
  HSI14InternOscAfterCalib = HSI14_CalibrateMinError();
  
#else   
  /* Calibrate HSI oscillator with the maximum allowed error in Hz */
  /* Fix the maximum value of the error frequency at +/- 40000Hz */
  HSI_CalibrateFixedError(40000, (uint32_t *)&InternOscAfterCalib);   
  
  /* Calibrate HSI14 oscillator with the maximum allowed error in Hz */
  /* Fix the maximum value of the error frequency at +/- 40000Hz */
  HSI14_CalibrateFixedError(40000, (uint32_t *)&HSI14InternOscAfterCalib);  
#endif
  
  /* Measure LSI oscillator frequency after HSI calibration */
  LSIFrequency = LSI_FreqMeasure(InternOscAfterCalib);
  
  /************************ Display results on the LCD ************************/
  LCD_Display();
  
  /* Infinite loop */
  while (1)
  {     
    /* Toggle LD1 */
    STM_EVAL_LEDToggle(LED1);
    
    /* Insert 50 ms delay */
    Delay(5);
    
    /* Toggle LD2 */
    STM_EVAL_LEDToggle(LED2);
    
    /* Insert 50 ms delay */
    Delay(5); 
  }
}

/**
  * @brief  Display LSI, HSI and HSI14 frequencies on the LCD.
  * @param  None.
  * @retval None
  */
static void LCD_Display(void)
{
  uint8_t hsidegit1 = 0, hsidegit2 = 0, hsidegit3 = 0, hsidegit4 = 0;
  uint8_t hsi14degit1 = 0, hsi14degit2 = 0, hsi14degit3 = 0;
  uint8_t lsidegit1 = 0, lsidegit2 = 0, lsidegit3 = 0;
  uint8_t TxBuffer[20];

  lsidegit1 = (LSIFrequency / 1000);
  lsidegit2 = (LSIFrequency % 1000) / 100;
  lsidegit3 = (LSIFrequency % 100) / 10;
  
  sprintf((char*)TxBuffer, "LSI Value = %d.%d%dKHz",lsidegit1, lsidegit2, lsidegit3);
  
  /* Set the LCD Back Color */
  LCD_SetBackColor(LCD_COLOR_BLUE2);
  
  /* Set the LCD Text Color */
  LCD_SetTextColor(LCD_COLOR_WHITE);
  
  /* Display the LSI oscillator frequency */
  LCD_DisplayStringLine(LCD_LINE_3, TxBuffer);
  
  /**************************************************************************/
  /* Display Calibration Results */
  /* Set the LCD Back Color */
  LCD_SetBackColor(LCD_COLOR_CYAN);
  
  /* Set the LCD Text Color */
  LCD_SetTextColor(LCD_COLOR_BLACK);
  
  /* Display Message on the LCD */
  LCD_DisplayStringLine(LCD_LINE_4, (uint8_t*)"Values before Calib.");
  
  hsidegit1 = (InternOscBeforeCalib / 1000000);
  hsidegit2 = (InternOscBeforeCalib % 1000000) / 100000;
  hsidegit3 = (InternOscBeforeCalib % 100000) / 10000;
  hsidegit4 = (InternOscBeforeCalib % 10000) / 1000;
  
  hsi14degit1 = (HSI14InternOscBeforeCalib / 1000000);
  hsi14degit2 = (HSI14InternOscBeforeCalib % 1000000) / 100000;
  hsi14degit3 = (HSI14InternOscBeforeCalib % 100000) / 10000;
  
  sprintf((char*)TxBuffer, "%d.%d%d%dMHz    %d.%d%dMHz",hsidegit1, hsidegit2,
           hsidegit3, hsidegit4, hsi14degit1, hsi14degit2, hsi14degit3);
  
  /* Set the LCD Back Color */
  LCD_SetBackColor(LCD_COLOR_BLUE);
  
  /* Set the LCD Text Color */
  LCD_SetTextColor(LCD_COLOR_WHITE);
  
  /* Display Message on the LCD */
  LCD_DisplayStringLine(LCD_LINE_5, (uint8_t*)"   HSI        HSI14 ");
  
  /* Display the LSI oscillator frequency */
  LCD_DisplayStringLine(LCD_LINE_6, TxBuffer);
  
  hsidegit1 = (InternOscAfterCalib / 1000000);
  hsidegit2 = (InternOscAfterCalib % 1000000) / 100000;
  hsidegit3 = (InternOscAfterCalib % 100000) / 10000;
  hsidegit4 = (InternOscAfterCalib % 10000) / 1000;
  
  hsi14degit1 = (HSI14InternOscAfterCalib / 1000000);
  hsi14degit2 = (HSI14InternOscAfterCalib % 1000000) / 100000;
  hsi14degit3 = (HSI14InternOscAfterCalib % 100000) / 10000;
  
  sprintf((char*)TxBuffer, "%d.%d%d%dMHz    %d.%d%dMHz",hsidegit1, hsidegit2, 
          hsidegit3, hsidegit4, hsi14degit1, hsi14degit2, hsi14degit3);
  
  /* Set the LCD Back Color */
  LCD_SetBackColor(LCD_COLOR_CYAN);
  
  /* Set the LCD Text Color */
  LCD_SetTextColor(LCD_COLOR_BLACK);
  
  /* Display Message on the LCD */
  LCD_DisplayStringLine(LCD_LINE_7, (uint8_t*)"Values after Calib. ");
  
  /* Set the LCD Back Color */
  LCD_SetBackColor(LCD_COLOR_BLUE);
  
  /* Set the LCD Text Color */
  LCD_SetTextColor(LCD_COLOR_WHITE);
  
  /* Display Message on the LCD */
  LCD_DisplayStringLine(LCD_LINE_8, (uint8_t*)"   HSI        HSI14 ");
  
  /* Display the LSI oscillator frequency */
  LCD_DisplayStringLine(LCD_LINE_9, TxBuffer);  
}

/**
  * @brief  Inserts a delay time.
  * @param  nTime: specifies the delay time length, in 10 ms.
  * @retval None
  */
void Delay(__IO uint32_t nTime)
{
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

/**
  * @brief  Decrements the TimingDelay variable.
  * @param  None
  * @retval None
  */
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/**
  * @}
  */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
